@@ -11,9 +11,12 @@ logger = logging.getLogger(__name__)
 
 DEFAULT_CAMERA_DEVICE_TYPES = [
     'PI3WITHCAMERA',
+    'PI4WITHCAMERA',
     'PIZEROWITHCAMERA'
 ]
 
+VIDEO_DURATION = datetime.timedelta(seconds=10)
+
 def fetch_videos(
     start=None,
     end=None,
@@ -27,7 +30,7 @@ def fetch_videos(
     camera_names=None,
     camera_serial_numbers=None,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -43,9 +46,9 @@ def fetch_videos(
     download_video_files(). See documentation of those functions for details.
 
     Args:
-        start (datetime): Earliest video start time (default is None)
-        end (datetime): Latest video start time (default is None)
-        video_timestamps (list of datetime): List of video start times (default is None)
+        start (datetime): Start of time period to fetch (default is None)
+        end (datetime): End of time period to fetch (default is None)
+        video_timestamps (list of datetime): List of video start times to fetch (default is None)
         camera_assignment_ids (list of str): Honeycomb assignment IDs (default is None)
         environment_id (str): Honeycomb environment ID (default is None)
         environment_name (str): Honeycomb environment name (default is None)
@@ -55,7 +58,7 @@ def fetch_videos(
         camera_names (list of str): Honeycomb device names (default is None)
         camera_serial_numbers (list of str): Honeycomb device serial numbers (default is None)
         chunk_size (int): Maximum number of data points to be returned by each Honeycomb query (default is 100)
-        minimal_honeycomb_client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
+        client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
         uri (str): Server URI for creating Honeycomb client (default is value of HONEYCOMB_URI environment variable)
         token_uri (str): Token URI for creating Honeycomb client (default is value of HONEYCOMB_TOKEN_URI environment variable)
         audience (str): Audience for creating Honeycomb client (default is value of HONEYCOMB_AUDIENCE environment variable)
@@ -67,7 +70,7 @@ def fetch_videos(
     Returns:
         (list of dict): Metadata for videos with local path information appended
     """
-    logging.info('Fetching metadata for videos that match specified parameters')
+    logger.info('Fetching metadata for videos that match specified parameters')
     video_metadata = fetch_video_metadata(
         start=start,
         end=end,
@@ -81,14 +84,14 @@ def fetch_videos(
         camera_names=camera_names,
         camera_serial_numbers=camera_serial_numbers,
         chunk_size=chunk_size,
-        minimal_honeycomb_client=minimal_honeycomb_client,
+        client=client,
         uri=uri,
         token_uri=token_uri,
         audience=audience,
         client_id=client_id,
         client_secret=client_secret
     )
-    logging.info('Downloading video files')
+    logger.info('Downloading video files')
     video_metadata_with_local_paths = download_video_files(
         video_metadata=video_metadata,
         local_video_directory=local_video_directory,
@@ -107,7 +110,7 @@ def fetch_images(
     camera_names=None,
     camera_serial_numbers=None,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -125,7 +128,7 @@ def fetch_images(
     download_image_files(). See documentation of those functions for details.
 
     Args:
-        image_timestamps (list of datetime): List of image timestamps
+        image_timestamps (list of datetime): List of image timestamps to fetch
         camera_assignment_ids (list of str): Honeycomb assignment IDs (default is None)
         environment_id (str): Honeycomb environment ID (default is None)
         environment_name (str): Honeycomb environment name (default is None)
@@ -135,7 +138,7 @@ def fetch_images(
         camera_names (list of str): Honeycomb device names (default is None)
         camera_serial_numbers (list of str): Honeycomb device serial numbers (default is None)
         chunk_size (int): Maximum number of data points to be returned by each Honeycomb query (default is 100)
-        minimal_honeycomb_client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
+        client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
         uri (str): Server URI for creating Honeycomb client (default is value of HONEYCOMB_URI environment variable)
         token_uri (str): Token URI for creating Honeycomb client (default is value of HONEYCOMB_TOKEN_URI environment variable)
         audience (str): Audience for creating Honeycomb client (default is value of HONEYCOMB_AUDIENCE environment variable)
@@ -149,7 +152,7 @@ def fetch_images(
     Returns:
         (list of dict): Metadata for images with local path information appended
     """
-    logging.info('Fetching metadata for images that match specified parameters')
+    logger.info('Fetching metadata for images that match specified parameters')
     image_metadata = fetch_image_metadata(
         image_timestamps=image_timestamps,
         camera_assignment_ids=camera_assignment_ids,
@@ -161,14 +164,14 @@ def fetch_images(
         camera_names=camera_names,
         camera_serial_numbers=camera_serial_numbers,
         chunk_size=chunk_size,
-        minimal_honeycomb_client=minimal_honeycomb_client,
+        client=client,
         uri=uri,
         token_uri=token_uri,
         audience=audience,
         client_id=client_id,
         client_secret=client_secret
     )
-    logging.info('Downloading image files')
+    logger.info('Downloading image files')
     image_metadata_with_local_paths = download_image_files(
         image_metadata=image_metadata,
         local_image_directory=local_image_directory,
@@ -191,7 +194,7 @@ def fetch_video_metadata(
     camera_names=None,
     camera_serial_numbers=None,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -210,14 +213,18 @@ def fetch_video_metadata(
     cannot specify environment name and environment ID, camera assignment IDs
     and camera device IDs, etc.)
 
+    If start and end are specified, returns all videos that overlap with
+    specified start and end (e.g., if start is 10:32:56 and end is 10:33:20,
+    returns videos starting at 10:32:50, 10:33:00 and 10:33:10).
+
     Returned metadata is a list of dictionaries, one for each video. Each
     dictionary has the following fields: data_id, video_timestamp,
     environment_id, assignment_id, device_id, bucket, key.
 
     Args:
-        start (datetime): Earliest video start time (default is None)
-        end (datetime): Latest video start time (default is None)
-        video_timestamps (list of datetime): List of video start times (default is None)
+        start (datetime): Start of time period to fetch (default is None)
+        end (datetime): End of time period to fetch (default is None)
+        video_timestamps (list of datetime): List of video start times to fetch (default is None)
         camera_assignment_ids (list of str): Honeycomb assignment IDs (default is None)
         environment_id (str): Honeycomb environment ID (default is None)
         environment_name (str): Honeycomb environment name (default is None)
@@ -227,7 +234,7 @@ def fetch_video_metadata(
         camera_names (list of str): Honeycomb device names (default is None)
         camera_serial_numbers (list of str): Honeycomb device serial numbers (default is None)
         chunk_size (int): Maximum number of data points to be returned by each Honeycomb query (default is 100)
-        minimal_honeycomb_client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
+        client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
         uri (str): Server URI for creating Honeycomb client (default is value of HONEYCOMB_URI environment variable)
         token_uri (str): Token URI for creating Honeycomb client (default is value of HONEYCOMB_TOKEN_URI environment variable)
         audience (str): Audience for creating Honeycomb client (default is value of HONEYCOMB_AUDIENCE environment variable)
@@ -239,6 +246,8 @@ def fetch_video_metadata(
     """
     if (start is not None or end is not None) and video_timestamps is not None:
         raise ValueError('Cannot specify start/end and list of video timestamps')
+    if video_timestamps is None and (start is None or end is None):
+        raise ValueError('If not specifying specific timestamps, must specify both start and end times')
     if (
         camera_assignment_ids is not None and
         (
@@ -260,18 +269,24 @@ def fetch_video_metadata(
     if environment_id is not None and environment_name is not None:
         raise ValueError('Cannot specify environment ID and environment name')
     if video_timestamps is not None:
-        start = min(video_timestamps)
-        end = max(video_timestamps)
-        video_timestamps_honeycomb = [minimal_honeycomb.to_honeycomb_datetime(video_timestamp) for video_timestamp in video_timestamps]
-    if start is not None:
-        start_honeycomb = minimal_honeycomb.to_honeycomb_datetime(start)
-    if end is not None:
-        end_honeycomb = minimal_honeycomb.to_honeycomb_datetime(end)
+        video_timestamps_utc = [video_timestamp.astimezone(datetime.timezone.utc) for video_timestamp in video_timestamps]
+        video_timestamp_min_utc = min(video_timestamps)
+        video_timestamp_max_utc = max(video_timestamps)
+        start_utc = video_timestamp_min_utc
+        end_utc = video_timestamp_max_utc + VIDEO_DURATION
+        video_timestamps_utc_honeycomb = [minimal_honeycomb.to_honeycomb_datetime(video_timestamp_utc) for video_timestamp_utc in video_timestamps_utc]
+    else:
+        start_utc = start.astimezone(datetime.timezone.utc)
+        end_utc = end.astimezone(datetime.timezone.utc)
+        video_timestamp_min_utc = video_timestamp_min(start_utc)
+        video_timestamp_max_utc = video_timestamp_max(end_utc)
+        start_utc_honeycomb = minimal_honeycomb.to_honeycomb_datetime(start_utc)
+        end_utc_honeycomb = minimal_honeycomb.to_honeycomb_datetime(end_utc)
     if environment_name is not None:
         environment_id = fetch_environment_id(
             environment_name=environment_name,
             chunk_size=chunk_size,
-            minimal_honeycomb_client=minimal_honeycomb_client,
+            client=client,
             uri=uri,
             token_uri=token_uri,
             audience=audience,
@@ -279,12 +294,12 @@ def fetch_video_metadata(
             client_secret=client_secret
         )
     camera_assignment_ids_from_environment = fetch_camera_assignment_ids_from_environment(
-        start=start,
-        end=end,
+        start=start_utc,
+        end=end_utc,
         environment_id=environment_id,
         camera_device_types=camera_device_types,
         chunk_size=chunk_size,
-        minimal_honeycomb_client=minimal_honeycomb_client,
+        client=client,
         uri=uri,
         token_uri=token_uri,
         audience=audience,
@@ -292,14 +307,14 @@ def fetch_video_metadata(
         client_secret=client_secret
     )
     camera_assignment_ids_from_camera_properties = fetch_camera_assignment_ids_from_camera_properties(
-        start=start,
-        end=end,
+        start=start_utc,
+        end=end_utc,
         camera_device_ids=camera_device_ids,
         camera_part_numbers=camera_part_numbers,
         camera_names=camera_names,
         camera_serial_numbers=camera_serial_numbers,
         chunk_size=100,
-        minimal_honeycomb_client=None,
+        client=None,
         uri=uri,
         token_uri=token_uri,
         audience=audience,
@@ -312,19 +327,19 @@ def fetch_video_metadata(
         query_list.append({
             'field': 'timestamp',
             'operator': 'GTE',
-            'value': start_honeycomb
+            'value': video_timestamp_min_utc
         })
     if end is not None:
         query_list.append({
             'field': 'timestamp',
             'operator': 'LTE',
-            'value': end_honeycomb
+            'value':video_timestamp_max_utc
         })
     if video_timestamps is not None:
         query_list.append({
             'field': 'timestamp',
             'operator': 'IN',
-            'values': video_timestamps_honeycomb
+            'values': video_timestamps_utc_honeycomb
         })
     if camera_assignment_ids is not None:
         query_list.append({
@@ -369,7 +384,7 @@ def fetch_video_metadata(
         query_list=query_list,
         return_data=return_data,
         chunk_size=chunk_size,
-        minimal_honeycomb_client=None,
+        client=None,
         uri=uri,
         token_uri=token_uri,
         audience=audience,
@@ -435,7 +450,7 @@ def download_video_files(
         if not os.path.exists(download_path):
             load_file_from_s3(video.get('key'), video.get('bucket'), download_path)
         else:
-            logging.info('File {} already exists'.format(download_path))
+            logger.info('File {} already exists'.format(download_path))
         video['video_local_path'] = download_path
         video_metadata_with_local_paths.append(video)
     return video_metadata_with_local_paths
@@ -463,7 +478,7 @@ def load_file_from_s3(
     download_path
 ):
     s3 = boto3.resource('s3')
-    logging.info('Loading {} from {} into {}'.format(
+    logger.info('Loading {} from {} into {}'.format(
         key,
         bucket_name,
         download_path
@@ -483,7 +498,7 @@ def fetch_image_metadata(
     camera_names=None,
     camera_serial_numbers=None,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -509,7 +524,7 @@ def fetch_image_metadata(
     device_id, bucket, key, and image_timestamp, and frame_number.
 
     Args:
-        image_timestamps (list of datetime): List of image timestamps
+        image_timestamps (list of datetime): List of image timestamps to fetch
         camera_assignment_ids (list of str): Honeycomb assignment IDs (default is None)
         environment_id (str): Honeycomb environment ID (default is None)
         environment_name (str): Honeycomb environment name (default is None)
@@ -519,7 +534,7 @@ def fetch_image_metadata(
         camera_names (list of str): Honeycomb device names (default is None)
         camera_serial_numbers (list of str): Honeycomb device serial numbers (default is None)
         chunk_size (int): Maximum number of data points to be returned by each Honeycomb query (default is 100)
-        minimal_honeycomb_client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
+        client (MinimalHoneycombClient): Existing Honeycomb client (otherwise will create one)
         uri (str): Server URI for creating Honeycomb client (default is value of HONEYCOMB_URI environment variable)
         token_uri (str): Token URI for creating Honeycomb client (default is value of HONEYCOMB_TOKEN_URI environment variable)
         audience (str): Audience for creating Honeycomb client (default is value of HONEYCOMB_AUDIENCE environment variable)
@@ -553,7 +568,7 @@ def fetch_image_metadata(
         camera_names=camera_names,
         camera_serial_numbers=camera_serial_numbers,
         chunk_size=chunk_size,
-        minimal_honeycomb_client=minimal_honeycomb_client,
+        client=client,
         uri=uri,
         token_uri=token_uri,
         audience=audience,
@@ -628,7 +643,7 @@ def download_image_files(
             os.makedirs(os.path.dirname(download_path), exist_ok=True)
             cv.imwrite(download_path, image_data)
         else:
-            logging.info('File {} already exists'.format(download_path))
+            logger.info('File {} already exists'.format(download_path))
         image['image_local_path'] = download_path
         image_metadata_with_local_paths.append(image)
     return image_metadata_with_local_paths
@@ -655,7 +670,7 @@ def image_local_path(
 def fetch_environment_id(
     environment_name=None,
     chunk_size=None,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -665,15 +680,15 @@ def fetch_environment_id(
     if environment_name is None:
         return None
     logger.info('Fetching environment ID for specified environment name')
-    if minimal_honeycomb_client is None:
-        minimal_honeycomb_client = minimal_honeycomb.MinimalHoneycombClient(
+    if client is None:
+        client = minimal_honeycomb.MinimalHoneycombClient(
             uri=uri,
             token_uri=token_uri,
             audience=audience,
             client_id=client_id,
             client_secret=client_secret
         )
-    result = minimal_honeycomb_client.bulk_query(
+    result = client.bulk_query(
         request_name='findEnvironments',
         arguments={
             'name': {
@@ -705,7 +720,7 @@ def fetch_camera_assignment_ids_from_environment(
     environment_id=None,
     camera_device_types=DEFAULT_CAMERA_DEVICE_TYPES,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -714,16 +729,21 @@ def fetch_camera_assignment_ids_from_environment(
 ):
     if environment_id is None:
         return None
-    logger.info('Fetching camera assignments for specified environment and time span')
-    if minimal_honeycomb_client is None:
-        minimal_honeycomb_client = minimal_honeycomb.MinimalHoneycombClient(
+    logger.info('Fetching camera assignments for environment {}, time span {} to {}, and camera device types {} '.format(
+        environment_id,
+        start.isoformat(),
+        end.isoformat(),
+        camera_device_types
+    ))
+    if client is None:
+        client = minimal_honeycomb.MinimalHoneycombClient(
             uri=uri,
             token_uri=token_uri,
             audience=audience,
             client_id=client_id,
             client_secret=client_secret
         )
-    result = minimal_honeycomb_client.request(
+    result = client.request(
         request_type='query',
         request_name='getEnvironment',
         arguments={
@@ -768,7 +788,7 @@ def fetch_camera_assignment_ids_from_camera_properties(
     camera_names=None,
     camera_serial_numbers=None,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -803,15 +823,15 @@ def fetch_camera_assignment_ids_from_camera_properties(
             'values': camera_serial_numbers
         })
     logger.info('Fetching camera assignments for cameras with specified properties')
-    if minimal_honeycomb_client is None:
-        minimal_honeycomb_client = minimal_honeycomb.MinimalHoneycombClient(
+    if client is None:
+        client = minimal_honeycomb.MinimalHoneycombClient(
             uri=uri,
             token_uri=token_uri,
             audience=audience,
             client_id=client_id,
             client_secret=client_secret
         )
-    result = minimal_honeycomb_client.bulk_query(
+    result = client.bulk_query(
         request_name='searchDevices',
         arguments={
             'query': {
@@ -851,7 +871,7 @@ def search_datapoints(
     query_list,
     return_data,
     chunk_size=100,
-    minimal_honeycomb_client=None,
+    client=None,
     uri=None,
     token_uri=None,
     audience=None,
@@ -859,15 +879,15 @@ def search_datapoints(
     client_secret=None
 ):
     logger.info('Searching for datapoints that match the specified parameters')
-    if minimal_honeycomb_client is None:
-        minimal_honeycomb_client = minimal_honeycomb.MinimalHoneycombClient(
+    if client is None:
+        client = minimal_honeycomb.MinimalHoneycombClient(
             uri=uri,
             token_uri=token_uri,
             audience=audience,
             client_id=client_id,
             client_secret=client_secret
         )
-    result = minimal_honeycomb_client.bulk_query(
+    result = client.bulk_query(
         request_name='searchDatapoints',
         arguments={
             'query': {
@@ -884,3 +904,35 @@ def search_datapoints(
     )
     logger.info('Fetched {} datapoints'.format(len(result)))
     return result
+
+def video_timestamp_min(start):
+    original_tzinfo = start.tzinfo
+    if original_tzinfo:
+        start_naive = start.astimezone(datetime.timezone.utc).replace(tzinfo=None)
+    else:
+        start_naive = start
+    timestamp_min_naive = (
+        datetime.datetime.min +
+        math.floor((start_naive - datetime.datetime.min)/VIDEO_DURATION)*VIDEO_DURATION
+    )
+    if original_tzinfo:
+        timestamp_min = timestamp_min_naive.replace(tzinfo=datetime.timezone.utc).astimezone(original_tzinfo)
+    else:
+        timestamp_min = timestamp_min_naive
+    return timestamp_min
+
+def video_timestamp_max(end):
+    original_tzinfo = end.tzinfo
+    if original_tzinfo:
+        end_naive = end.astimezone(datetime.timezone.utc).replace(tzinfo=None)
+    else:
+        end_naive = end
+    timestamp_max_naive = (
+        datetime.datetime.min +
+        (math.ceil((end_naive - datetime.datetime.min)/VIDEO_DURATION) - 1)*VIDEO_DURATION
+    )
+    if original_tzinfo:
+        timestamp_max = timestamp_max_naive.replace(tzinfo=datetime.timezone.utc).astimezone(original_tzinfo)
+    else:
+        timestamp_max = timestamp_max_naive
+    return timestamp_max
